#include "gui/menu/menu.hpp"
#include <GL/glew.h>
#include <GL/gl.h>
#include <fstream>
#include <nlohmann/json.hpp>
#include "imgui.h"
#include "gui/menu/menu_data.hpp"
#include "character_sheet.hpp"
#include "ImFileDialog.h"

gui::menu::Menu::Menu(gui::menu::MenuData &data) noexcept: data(data) {
	SPDLOG_DEBUG("Creating Menu");
	ifd::FileDialog::Instance().CreateTexture = [](uint8_t *data, int w, int h, char fmt) -> void * {
		GLuint tex;

		glGenTextures(1, &tex);
		glBindTexture(GL_TEXTURE_2D, tex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, (fmt == 0) ? GL_BGRA : GL_RGBA, GL_UNSIGNED_BYTE, data);
		glGenerateMipmap(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, 0);

		return (void *) tex;
	};

	ifd::FileDialog::Instance().DeleteTexture = [](void *tex) -> void {
		auto texID = (GLuint) ((std::uintptr_t) tex);
		glDeleteTextures(1, &texID);
	};
}

void gui::menu::Menu::gui() const noexcept {
	if (ImGui::BeginMenuBar()) {
		if (ImGui::BeginMenu("File")) {
			if (ImGui::MenuItem("Open file")) {
				// TODO
			}

			if (ImGui::MenuItem("Save")) {
				std::ofstream file{ data.save_path / data.filename };
				nlohmann::json j;
				j.emplace("character_sheet", data.character_sheet);
			}

			if (ImGui::MenuItem("Save as...")) {
				// TODO
			}

			ImGui::EndMenu();
		}
		if (ImGui::BeginMenu("Editor")) {
			ImGui::MenuItem("Edit mode", nullptr, &data.edit_mode);
			ImGui::EndMenu();
		}
		ImGui::EndMenuBar();
	}
}
